@@ -75,8 +75,80 @@ typedef struct{
 
 ihost_config_t ihost_config;
 
+extern int errno;
+
+/* Taken from the OpenSSH code. Its licence included in function.*/
+#ifndef HAVE_STRLCAT
+
+/*
+ * Copyright (c) 1998 Todd C. Miller <Todd.Miller@courtesan.com>
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. The name of the author may not be used to endorse or promote products
+ *    derived from this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
+ * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
+ * THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
+ * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
+ * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
+ * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ */
+
+/*
+ * Appends src to string dst of size siz (unlike strncat, siz is the
+ * full size of dst, not space left).  At most siz-1 characters
+ * will be copied.  Always NUL terminates (unless siz <= strlen(dst)).
+ * Returns strlen(src) + MIN(siz, strlen(initial dst)).
+ * If retval >= siz, truncation occurred.
+ */
+size_t
+strlcat(dst, src, siz)
+        char *dst;
+        const char *src;
+        size_t siz;
+{
+        register char *d = dst;
+        register const char *s = src;
+        register size_t n = siz;
+        size_t dlen;
+
+        /* Find the end of dst and adjust bytes left but don't go past end */
+        while (n-- != 0 && *d != '\0')
+                d++;
+        dlen = d - dst;
+        n = siz - dlen;
+
+        if (n == 0)
+                return(dlen + strlen(s));
+        while (*s != '\0') {
+                if (n != 1) {
+                        *d++ = *s;
+                        n--;
+                }
+                s++;
+        }
+        *d = '\0';
+
+        return(dlen + (s - src));       /* count does not include NUL */
+}
+
+#endif
+/* End strlcat function taken from OpenSSH */
+
 void log_msg(int level, char *format, ...){
-	extern int errno;
 	int cur_errno;
 	va_list ap;
 
@@ -91,6 +163,7 @@ void log_msg(int level, char *format, ...){
 		}else{
 			fprintf(ihost_config.log, "\n");
 		}
+		fflush(ihost_config.log);
 	}
 }
 
@@ -150,6 +223,7 @@ FILE *create_tcp_connection(char *hostname, int port){
 
 	if((get_host_addr(hostname, &haddr))!=0){
 		log_msg(LOG_CRIT, "Failed to lookup name for %s", hostname);
+		close(sock);
 		return NULL;
 	}
 
@@ -161,6 +235,7 @@ FILE *create_tcp_connection(char *hostname, int port){
 	log_msg(LOG_DEBUG, "Creating a tcp connection");
 	if(connect(sock, (struct sockaddr *)&addr, sizeof(addr)) !=0){
 		log_msg(LOG_CRIT, "Failed to connect to hostname %s on port %d", hostname, port);
+		close(sock);
 		return NULL;
 	}
 
@@ -220,7 +295,9 @@ int ihost_getconfig(ihost_state_t *ihost_state){
 	int server_udp_port=0;
 	time_t config_ttl=0;
 
-	tcp_con=create_tcp_connection(ihost_state->filtermanager_host, ihost_state->filtermanager_port);
+	if((tcp_con=create_tcp_connection(ihost_state->filtermanager_host, ihost_state->filtermanager_port))==NULL){
+		return -1;
+	}
 
 	if(ihost_state->file_list!=NULL || ihost_state->last_modified!=NULL){
 		if(tcp_con==NULL){
@@ -236,7 +313,7 @@ int ihost_getconfig(ihost_state_t *ihost_state){
 		}
 	
 		if((tcp_comm(tcp_con, ihost_state->last_modified, &response, "OK"))==0){
-			if((tcp_comm(tcp_con, "END", &response, "OK"))==0){
+			if((tcp_comm(tcp_con, "END", &response, "OK"))!=0){
 				goto error;
 			}
 			fclose(tcp_con);
@@ -504,7 +581,7 @@ int get_system_stats(int seq_no, ihost_state_t *ihost_state, char *xml, int size
 			y=page_stats->pages_pageout;
 		}
 		snprintf(tmp, size, \
-			"<pages><swapins>%lld</swapins><swapouts>%lld</swapouts></pages>", \
+			"<pages><pageins>%lld</pageins><pageouts>%lld</pageouts></pages>", \
 			x, \
 			y);
 	
